# Importing the libraries
from .utils.alphanumeric_sorter_class import AlphanumericSorter
from .utils.file_type_checker_class import FileTypeChecker
from .main.augmentations_class import Augmentations
from .main.video_augmentor_class import VideoAugmentor
from .main.image_augmentor_class import ImageAugmentor



# Importing the libraries
from .assets.utils.alphanumeric_sorter_class import AlphanumericSorter
from .assets.utils.file_type_checker_class import FileTypeChecker
from .assets.main.video_augmentor_class import VideoAugmentor
from .assets.main.image_augmentor_class import ImageAugmentor
from typing import Dict, Callable, Optional
import os


# Defining the Pipeline class
class Pipeline(AlphanumericSorter, FileTypeChecker):

    # Defining the constructor
    def __init__(self) -> None:

        """

        Constructor of the Pipeline class.
        Initializes the AlphanumericSorter, ImageAugmentor and VideoAugmentor classes.

        
        Parameters
        ----------
        None.

        
        Returns
        -------
        None.
        
        """

        pass


    # Defining the augment method
    def augment(self, input_path: str, output_path: str, target: str, type: str, mode: str,
                augmentations: Dict[str, Callable], verbose: Optional[bool] = False, warn_verbose: Optional[bool] = False) -> None:
        
        """
        
        Augments the input data.


        Parameters
        ----------
        input_path : str
            Path to the input data.

        output_path : str
            Path to the output data.

        target : str
            Target of the augmentation.
                The options are:
                    "image"
                        Specifies that the target of the augmentation is an image.
                    "video"
                        Specifies that the target of the augmentation is a video.

        type : str
            Type of the augmentation.
                The options are:
                    "batch"
                        Batch type means that the input and output paths are directories.
                    "single"
                        Single type means that the input and output paths are files.

        mode : str
            Mode of the augmentation.
                The options are:
                    "sequential"
                        Sequential mode means that the augmentations will be applied one by one.
                        This means that the data will be saved after each augmentation.
                    "singular"
                        Singular mode means that the augmentations will be applied all at once.
                        This means that the data will be saved only after all augmentations were applied.

        augmentations : dict
            Dict of augmentations to be applied to the data. 
            The keys are the names of the augmentations and the values are the augmentations themselves.

        verbose : bool, optional
            If True, prints the progress of the augmentation process. The default is False.

        
        Returns
        -------
        None.
        
        """

        # Checking if the input path is valid
        if not os.path.exists(input_path):
            raise ValueError(f"Invalid input path: {input_path}")
        
        # Checking if the type is valid
        if type != "batch" and type != "single":
            raise ValueError(f"Invalid type type: {type}")
        
        # Checking if the output path is valid
        if type == "batch" and not os.path.exists(output_path):
            raise ValueError(f"Invalid output path: {output_path}")

        # Checking if the target is valid
        if target != "image" and target != "video":
            raise ValueError(f"Invalid target type: {target}")
        
        # Checking if the mode is valid
        if mode != "sequential" and mode != "singular":
            raise ValueError(f"Invalid mode type: {mode}")
        
        # Checking if the augmentations is valid
        if not isinstance(augmentations, dict):
            raise ValueError(f"Invalid augmentations type: {type}")
        elif len(augmentations) == 0:
            raise ValueError("augmentations dict is empty.")
        
        # Checking if the verbose is valid
        if not isinstance(verbose, bool):
            raise ValueError(f"Invalid verbose type: {verbose}")


        # Choosing the augmentor
        if target == "image":
            augmentor = ImageAugmentor()
        elif target == "video":
            augmentor = VideoAugmentor()
        else:
            raise ValueError(f"Invalid target type: {target}")
        
        # Augmentation
        if type == "batch":
            
            # Looping through the input files
            for input_file in self.sorted_alphanumeric(os.listdir(input_path)):
                
                # Checking if the input file is valid
                if not self.is_target_type(input_file, target):

                    if warn_verbose:
                        print(f"Skipping invalid input file: {input_file}")

                    continue

                # Defining the input file path
                input_file_path = os.path.join(input_path, input_file)

                # Defining the output file path
                output_file_path = os.path.join(output_path, input_file)

                # Augmenting the input file
                if mode == "sequential":
                    augmentor.augment_one_by_one(input_file_path, output_file_path, augmentations, verbose)
                elif mode == "singular":
                    augmentor.augment_all_at_once(input_file_path, output_file_path, augmentations, verbose)

        elif type == "single":
                
                # Checking if the input file is valid
                if not self.is_target_type(input_path, target):
                    raise ValueError(f"Invalid input file. Expected {target} file, got {os.path.splitext(input_path)[1]} instead.")
                
                # Checking if the format of the input file and the output file are the same
                if os.path.splitext(input_path)[1] != os.path.splitext(output_path)[1]:
                    raise ValueError(f"Input file format and output file format are not the same. Got {os.path.splitext(input_path)[1]} in input, and {os.path.splitext(output_path)[1]} in output.")
                elif os.path.splitext(input_path)[1] == ".gif":
                    raise ValueError(f"The GIF format is not supported.")
                
                # Augmenting the input file
                if mode == "sequential":
                    augmentor.augment_one_by_one(input_path, output_path, augmentations, verbose)
                elif mode == "singular":
                    augmentor.augment_all_at_once(input_path, output_path, augmentations, verbose)



# Import the libraries
import mimetypes


# Defining the FileChecker class
class FileTypeChecker():

    """
    
    Class that checks the file type.
    
    """

    # Defining the constructor
    def __init__(self) -> None:
            
        """

        Constructor of the FileChecker class.

        
        Parameters
        ----------
        None.

        
        Returns
        -------
        None.
        
        """

        pass


    # Defining the is_target_type method
    def is_target_type(self, filename: str, target: str) -> bool:

        """

        Checks if the file is of the target type.


        Parameters
        ----------
        filename : str
            Name of the file.

        target : str
            Target type.

        
        Returns
        -------
        bool
            True if the file is of the target type, False otherwise.

        """

        # Get the mime type
        mime_type, _ = mimetypes.guess_type(filename)


        # Return True if the mime type is not None and starts with the target type
        return mime_type is not None and mime_type.startswith(f'{target}/')


# Importing the libraries
import re


# Defining the alphanumeric sorter class
class AlphanumericSorter():

    # Defining the constructor
    def __init__(self) -> None:

        '''
        
        Parameters
        ----------
        None.

        
        Returns
        -------
        None.

        '''
        
        pass


    # Sorting the list alphanumerically
    def sorted_alphanumeric(self, data: list) -> list:

        '''
        
        Parameters
        ----------
        data : list

        
        Returns
        -------
        sorted_data : list

        '''
        
        convert = lambda text: int(text) if text.isdigit() else text.lower()
        alphanum_key = lambda key: [convert(c) for c in re.split('([0-9]+)', key)] 


        return sorted(data, key=alphanum_key)


# Importing the libraries
from setuptools import setup


# Defining the readme function
def readme():
    with open('README.md') as f:
        return f.read()
    

# Defining the setup function
setup(name='CVAugmentor',
        version='0.0.1',
        description='A package for augmenting images and videos for computer vision tasks',
        long_description=readme(),
        long_description_content_type='text/markdown',
        url='https://github.com/AliKHaliliT/CVAugmentor',
        author='Ali Khalili Tazehkandgheshlagh',
        author_email='ali.khalili.t98@gmail.com',
        license='MIT',
        classifiers=[
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.9",
            "Operating System :: OS Independent",
        ],
        packages=['CVAugmentor', 'CVAugmentor.assets', 'CVAugmentor.assets.utils', 'CVAugmentor.assets.main'],
        include_package_data=True,
)


# Importing the libraries
from typing import Dict, Callable, Optional
from PIL import Image
import os


# Defining the ImageAugmentor class
class ImageAugmentor():
    
    def __init__(self) -> None:

        """

        Constructor of the ImageAugmentor class.
        
        
        Parameters
        ----------
        None.

        
        Returns
        -------
        None.
        
        """
        
        pass


    def augment_one_by_one(self, image_path: str, output_path: str, augmentations: Dict[str, Callable], verbose: Optional[bool] = False) -> None:

        """

        Augments the image one augmentation at a time.
        This means that the image will be augmented by the first augmentation, then it will be saved. 
        Next the image will be augmented by the second augmentation, then it will be saved. And so on.

        
        Parameters
        ----------
        image_path : str
            Path to the image file to be augmented.
            
        output_path : str
            Path to the output image file.

        augmentations : dict
            Dict of augmentations to be applied to the image. 
            The keys are the names of the augmentations and the values are the augmentations themselves.

        verbose : bool, optional
            If True, prints the progress of the augmentation process. The default is False.

        
        Returns
        -------
        None.
        
        """

        # Applying the augmentations
        for name, augmentation in augmentations.items():

            # Loading the image
            image = Image.open(image_path)

            # Applying the augmentation
            image = augmentation(image)

            # Saving the augmented image
            image.save(os.path.splitext(output_path)[0] + '_' + name + os.path.splitext(image_path)[1])

            # Printing the progress
            if verbose:
                print('Applied the ' + name + ' augmentation.')


        # Print a message
        if verbose:
            print('Augmentation completed.')


    def augment_all_at_once(self, image_path: str, output_path: str, augmentations: Dict[str, Callable], verbose: Optional[bool] = False) -> None:

        """

        Augments the image all at once.
        This means that the image will be augmented by all the augmentations at once, then it will be saved.

        
        Parameters
        ----------
        image_path : str
            Path to the image file to be augmented.

        output_path : str
            Path to the output image file.

        augmentations : dict
            Dict of augmentations to be applied to the image. 
            The keys are the names of the augmentations and the values are the augmentations themselves.

        verbose : bool, optional
            If True, prints the progress of the augmentation process. The default is False.

        
        Returns
        -------
        None.
        
        """

        # Loading the image
        image = Image.open(image_path)

        # Applying the augmentations
        for _, augmentation in augmentations.items():

            # Applying the augmentation
            image = augmentation(image)

        # Saving the augmented image
        image.save(output_path)


        # Print a message
        if verbose:
            print('Augmentation completed.')